﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Threading;

namespace ThreadTestCSAsync
{
    public partial class Form1 : Form
    {
        private int count = 0;
        private bool exec = false;
        private CancellationTokenSource canceled;
        private TaskCompletionSource<bool> waitForStop;
        protected override void WndProc(ref Message m)  // 実行中はクローズボタンを無効にする
        {
            const int WM_SYSCOMMAND = 0x112;
            const long SC_CLOSE = 0xF060L;
            if (exec == true && m.Msg == WM_SYSCOMMAND && (m.WParam.ToInt64() & 0xFFF0L) == SC_CLOSE)
                return;
            base.WndProc(ref m);
        }
        public Form1()
        {
            InitializeComponent();
            MaximizeBox = false;
        }
        private async void button1_Click(object sender, EventArgs e)
        {
            if (canceled != null)
                return;
            ClearCount();
            button1.Enabled = false;
            button2.Enabled = true;
            exec = true;
            canceled = new CancellationTokenSource();
            try
            {
                int i;
                for (i = 1; i <= 10; i++)
                {
                    IncrementCount();
                    await Task.Run(() =>
                    {
                        Thread.Sleep(3000);
                    });
                    if (waitForStop != null)
                        await waitForStop.Task;
                    canceled.Token.ThrowIfCancellationRequested();
                }
                MessageBox.Show("正常終了", "情報", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (OperationCanceledException)
            {
                ClearCount();
                MessageBox.Show("中止", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception)
            {
                ClearCount();
                MessageBox.Show("異常終了", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                canceled = null;
                button1.Enabled = true;
                button2.Enabled = false;
                exec = false;
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            if (waitForStop != null)
                return;
            waitForStop = new TaskCompletionSource<bool>();
            try
            {
                if (MessageBox.Show("中止しますか？", "質問", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    if (canceled != null)
                    {
                        canceled.Cancel();
                        button2.Enabled = false;
                    }
                }
            }
            finally
            {
                waitForStop.SetResult(true);
                waitForStop = null;
            }
        }
        private void ClearCount()
        {
            count = 0;
            UpdateText();
        }
        private void IncrementCount()
        {
            count = count + 1;
            UpdateText();
        }
        private void UpdateText()
        {
            label1.Text = string.Format("時間のかかる処理を実行中。{0}/10", count);
        }
    }
}
